import re
import pytest
from gepref_text.regex import (
        RegexRemovalStep, UrlRemovalStep, SpRemovalStep,
        NumRemovalStep, DupSpacesStep
        )

class TestRegexRemoval:
    @pytest.mark.parametrize("text, pattern, result", [
        ("The 333 text", " 3{3,3} ", "The text"),
        ("absolute absurd", "abs", " olute  urd"),
        ])
    def test_patterns(self, text: str, pattern: str, result: str):
        pat = re.compile(pattern)
        norm_text = RegexRemovalStep().set_pattern(pat)(text)
        assert norm_text == result

    @pytest.mark.parametrize("text, replace_str, result", [
        ("the 123 is awesome", "man", "the man is awesome"),
        ("this 12356 the best", "is", "this is the best")
        ])
    def test_replace(self, text: str, replace_str: str, result: str):
        pat = re.compile(r"\d+")
        norm_text = (
                RegexRemovalStep()
                .set_pattern(pat)
                .set_replace_str(replace_str)(text)
                )
        assert norm_text == result

@pytest.mark.parametrize("text, result", [
    ("find the data https://amazingurl.com here", "find the data   here"),
    ("the http://amazingurl.com", "the  "),
    ("http://thisisit.es", " ")
    ])
def test_url_removal(text: str, result: str):
    norm_text = UrlRemovalStep()(text)
    assert norm_text == result

@pytest.mark.parametrize("text, result", [
    ("t*h-e@ messag.,_e", "the message"),
    ("1!2@3#4$5%6^7&8*9(0)", "1234567890"),
    ])
def test_sp_removal(text: str, result: str):
    norm_text = SpRemovalStep().set_replace_str('')(text)
    assert norm_text == result

@pytest.mark.parametrize("text, result", [
    ("this1is2it", "this is it"),
    ("12345abcd67809", " abcd "),
    ])
def test_num_removal(text: str, result: str):
    norm_text = NumRemovalStep()(text)
    assert norm_text == result

@pytest.mark.parametrize("text, result", [
    ("hello    world", "hello world"),
    ("the\n test", "the test"),
    ])
def test_dupspaces_removal(text: str, result: str):
    norm_text = DupSpacesStep()(text)
    assert norm_text == result



import re
from abc import ABC, abstractmethod
from typing import Optional
from gepref_text.base import AbstractTextStep

class AbstractRegexSubStep(AbstractTextStep, ABC):
    """
    Interface that defines any regex substitution preprocessing step.
    The `get_pattern` method must be implemented.

    :param pattern: Regex pattern to match.
    :type pattern: re.Pattern
    :param replace_str: String to replace.
    :type replace_str: str
    """

    pattern: Optional[re.Pattern] = None
    replace_str: str = ' '

    def set_replace_str(self, replace_str: str) -> "AbstractRegexSubStep":
        """
        Setter for the `replace_str` parameter.

        :param replace_str: String to replace
        :type replace_str: str
        :returns: Instance of the step
        :rtype: AbstractRegexSubStep
        """
        self.replace_str = replace_str
        return self

    def set_pattern(self, pattern: re.Pattern) -> "AbstractRegexSubStep":
        """
        Setter for the `pattern` parameter.

        :param pattern: Regex pattern to match.
        :type pattern: re.Pattern
        :returns: Instance of the step
        :rtype: AbstractRegexSubStep
        """
        self.pattern = pattern
        return self

    def call(self, data: str) -> str:
        """
        Preprocess an input text element.

        :param data: Input text element.
        :type data: str
        :returns: Preprocessed data.
        :rtype: str
        """
        if self.pattern is None:
            self.pattern = self.get_pattern()
        return re.sub(self.pattern, self.replace_str, data)

    @abstractmethod
    def get_pattern(self) -> re.Pattern:
        ...

class RegexRemovalStep(AbstractRegexSubStep):
    """
    Step to replace a given text using a regex pattern. You can specify

    :param pattern: Regex pattern to match.
    :type pattern: re.Pattern
    :param replace_str: String to replace.
    :type replace_str: str
    """
    def get_pattern(self) -> re.Pattern:
        return re.compile(r"")

class UrlRemovalStep(AbstractRegexSubStep):
    """
    Step to remove an http or https url from text.

    :param pattern: Regex pattern to match.
    :type pattern: re.Pattern
    :param replace_str: String to replace.
    :type replace_str: str
    """
    def get_pattern(self) -> re.Pattern:
        return re.compile(r"https?://[^\s]+")

class SpRemovalStep(AbstractRegexSubStep):
    """
    Step to remove special characters (not letters or numbers) from text.

    :param pattern: Regex pattern to match.
    :type pattern: re.Pattern
    :param replace_str: String to replace.
    :type replace_str: str
    """
    def get_pattern(self) -> re.Pattern:
        return re.compile(r"[^a-zA-Z0-9 ]")

class NumRemovalStep(AbstractRegexSubStep):
    """
    Step to remove numbers from text.

    :param pattern: Regex pattern to match.
    :type pattern: re.Pattern
    :param replace_str: String to replace.
    :type replace_str: str
    """
    def get_pattern(self) -> re.Pattern:
        return re.compile(r"\d+")

class DupSpacesStep(AbstractRegexSubStep):
    """
    Step to remove duplicated spaces.

    :param pattern: Regex pattern to match.
    :type pattern: re.Pattern
    :param replace_str: String to replace.
    :type replace_str: str
    """
    def get_pattern(self) -> re.Pattern:
        return re.compile(r"\s+")
